# Load required libraries

library(ggplot2)
library(tidyr)
library(scales)
library(gridExtra)
library(h2o)
library(caret)
library(ggplot2)
library(dplyr)
library(ReactomePA)

#Change directory to were the csv with the DCt-values resides
setwd("XXXX")
data_pred<- read.csv("for_ml.csv", header=T, as.is=T, check.names=FALSE)

d2_pred <- 1/data_pred[,-1]
d2_pred$class<-data_pred$class


#h2o


# start h2o cluster
invisible(h2o.init())
# convert data as h2o type

data_h = as.h2o(d2_pred) 


# Set the seed for reproducibility
set.seed(1337)

# Indices for the 60/40 split, ensuring both classes in each split
indices <- createDataPartition(d2_pred$class, p = 0.6, list = FALSE, groups = 2)

# Create the 60% and 40% splits
train_d2 <- d2_pred[indices, ]
test_d2 <- d2_pred[-indices, ]



library(gt)
library(tidyverse)
test_d2 %>% 
  gt() %>% 
  cols_align("center")


# set label type

train_h = as.h2o(train_d2) 
test_h = as.h2o(test_d2) 


y = 'class'
pred = setdiff(names(train_h), y)
#convert variables to factors
train_h[,y] = as.factor(train_h[,y])
test_h[,y] = as.factor(test_h[,y])

# Run AutoML for 20 base models

gbm_params <- list(
  min_rows = 10
)


aml = h2o.automl(x = pred, y = y,
                 training_frame = train_h,
                 max_models = 50,
                 seed = 1337,
                 max_runtime_secs = 600,
                 nfolds=10,
                 exclude_algos = c("StackedEnsemble","DeepLearning")
)

?h2o.automl


# AutoML Leaderboard
lb = aml@leaderboard
a<-as.data.frame(lb)

write.csv(a,"leaderboard_pred.csv")


best_model <- aml@leader
perf <- h2o.performance(best_model, newdata = test_h)



top_models <- lb[!duplicated(lb$algo), ]

# Select the top 5 models
top_5_models <- top_models[1:5, ]
# Updated function to get variable importances for a given model ID and keep only top 10 variables
get_varimp <- function(model_id) {
  model <- h2o.getModel(model_id)
  varimp <- h2o.varimp(model)
  varimp_df <- as.data.frame(varimp)
  # Keep only the top 10 important variables
  varimp_df <- head(varimp_df, 10)
  # Add the model ID as a new column
  varimp_df$Model_ID <- model_id
  perf <- h2o.performance(model, newdata = test_h)
  
  
  # Add AUC and RMSE information
  varimp_df$AUC <- h2o.auc(perf)
  varimp_df$RMSE <- h2o.rmse(perf)
  
  return(varimp_df)
}


# Get variable importances for the top 5 models
varimp_list <- lapply(top_5_models$model_id, get_varimp)

# Combine variable importances into a single data frame
all_varimp <- bind_rows(varimp_list)

# Plot variable importances, AUC, and RMSE for the top 5 models in a single panel
models_pred<-ggplot(all_varimp, aes(x = reorder(variable, -percentage), y = percentage, fill = as.factor(Model_ID))) +
  geom_bar(stat = "identity", position = "dodge") +
  labs(title = "Top 5 Models - Top 10 Variable Importances", x = "Variable", y = "Percentage") +
  theme(axis.text.x = element_text(angle = 45, hjust = 1), legend.position = "none") +
  scale_fill_discrete(name = "Model ID") +
  facet_wrap(~ Model_ID, scales = "free", ncol = 1, labeller = labeller(Model_ID = function(x) {
    auc <- unique(all_varimp$AUC[all_varimp$Model_ID == x])
    rmse <- unique(all_varimp$RMSE[all_varimp$Model_ID == x])
    paste(x, "\nAUC: ", round(auc, 2), " RMSE: ", rmse)
  }))

#Change filename according to the analysis
ggsave("prediction_models.pdf", models_pred, width = 14, height = 10)

# Extract the top 10 variable names for each model
top_10_variables <- lapply(varimp_list, function(varimp_df) {
  varimp_df$variable
})

# Find the intersection of the top 10 variables for all models
intersection_top_10 <- Reduce(intersect, top_10_variables)

# Find sub-intersections of at least 4 models
sub_intersections <- list()
num_models <- length(top_10_variables)
for (i in 1:(num_models - 3)) {
  for (j in (i + 1):(num_models - 2)) {
    for (k in (j + 1):(num_models - 1)) {
      for (l in (k + 1):num_models) {
        sub_intersection <- intersect(intersect(intersect(top_10_variables[[i]], top_10_variables[[j]]), top_10_variables[[k]]), top_10_variables[[l]])
        if (length(sub_intersection) >= 1) {
          sub_intersections[[paste(i, j, k, l, sep = "_")]] <- sub_intersection
        }
      }
    }
  }
}

# Print the sub-intersections
print(sub_intersections)


sub_intersections_df <- data.frame(do.call(rbind, sub_intersections))

# Transpose the data frame to have sub-intersections as rows and variables as columns
sub_intersections_df <- as.data.frame(t(sub_intersections_df))
write.csv(sub_intersections_df, file = "sub_intersections_pred.csv", row.names = FALSE)



h2o.shutdown(prompt = F)